'
' Use an ADC channel to create a random bit, then use this bit to build random bytes or words
' This demo is a lottery number predictor
'
' If it produces the winning numbers, please remember me!
'
' Uses AN0 as the ADC channel for the random bit generator
'
' Author Les Johnson 02-02-2012 
'
' You are free to use this code as you see fit, however, a mention of the author and compiler would be appreciated.
' The code is offered as-is and Crownhill nor the author take any responsibility for its use.
'
    Include "Amicus18.inc"                  ' Configure for the Amicus18 device (18F25K20) running at 64MHz
    Include "Amicus18_ADC.inc"              ' Load the Amicus18 ADC routines into the program

$define Random_ADC_Channel 0                ' Use AN0 as the random bit generator ADC channel    
    Include "Real_Random.inc"               ' Load the Real Random macros into the program

    Dim wRandomResult As Word
    Dim bRandomResult As wRandomResult.Byte0

    Dim tDuplicateFound As Bit              ' True if a duplicate value is found in the random results
    Dim Predict As Byte
    Dim Index As Byte
    Dim WinningNumbers[7] As Byte           ' Holds the seven predicted numbers

'----------------------------------------------------------------------------------------
    GoTo Main                                           ' Jump over the subroutines
'----------------------------------------------------------------------------------------
' Make sure any numbers predicted are not duplicated
' Input     : Array WinningNumbers holds the values already created
' Output    : tDuplicateFound will hold True (1) if duplicate value found
' Notes     : None
'
CheckForDuplicate:
    tDuplicateFound = False                             ' Default to no duplicate found
    For Index = 0 To 6                                  ' Scan all 7 elements of array
        If bRandomResult = WinningNumbers[Index] Then   ' Is the number already in the array?
            tDuplicateFound = True                      ' Yes. So exit true
            Break
        EndIf
    Next
    Return
'----------------------------------------------------------------------------------------
' Create a random number between 1 and 49
' Input     : None
' Output    : bRandomResult holds the random number between 1 and 49
' Notes     : None
'
Get_WinningNumber:
    Repeat
        Random_Get(100, bRandomResult)                  ' Create a random value between 0 to 99
    Until bRandomResult > 0 And bRandomResult < 50      ' Keep looking until the value falls between 1 and 49
    Return
'---------------------------------------------------------
' The main program loop starts here
' Press a key on the serial terminal to start the prediction of 7 numbers
' Each number will be displayed on the serial terminal
'
Main:
    Clear                                               ' Clear all user RAM
    HRSOut "     Proton Lottery Predictor!\r"
    Random_Get(1000, wRandomResult)                     ' \
    DelayMS wRandomResult                               ' / Delay a random amount of time before starting
    
    HRSOut "Press a key to start the prediction\r\r"
    WREG = HRSIn                                        ' Wait for a serial character
    Random_Get(1000, wRandomResult)                     ' \
    DelayMS wRandomResult                               ' / Pause for drama
    HRSOut 13
    For Predict = 0 To 6                                ' Create a loop for 7 numbers
        Repeat
            GoSub Get_WinningNumber                     ' Get a random number
            GoSub CheckForDuplicate                     ' Have we got a duplicated number?
        Until tDuplicateFound = False                   ' Keep looking until not duplicated
        WinningNumbers[Predict] = bRandomResult         ' Load the array with the predicted number

        If Predict = 6 Then                             ' Is this the bonus ball?
            HRSOut "\rAnd the Bonus Number is: "
            DelayMS 5000                                ' Pause for drama
            HRSOut "(", Dec WinningNumbers[Predict], ")", 13
        Else
            HRSOut "The Winning number is:   (", Dec WinningNumbers[Predict], ")", 13   ' Display each number predicted
        EndIf
        DelayMS 1000                                    ' Pause for drama
        Random_Get(500, wRandomResult)
        DelayMS wRandomResult
    Next
    HRSOut "\r\rGood Luck!\r"
    


