$ifndef _REAL_RANDOM_INC_
$define _REAL_RANDOM_INC_
' Produce really random values from an open ADC channel on the device.
'
' Written for the Crownhill Proton compiler version 3.5.3.0 onwards using an 18F device
'
' Author Les Johnson 02-02-2012 
'
' You are free to use this code as you see fit, however, a mention of the author and compiler would be appreciated.
' The code is offered as-is and Crownhill nor the author take any responsibility for its use.
' 
Declare Reminders = Off                         ' Disable the compiler's reminder messages
'
' Use an unused ADC channel to create a random bit, then use this bit to build random bytes or words
' The ADC channel is chosen using a define, but defaults to AN0
'
    Include "Amicus18_ADC.inc"                           ' Load the ADC macros into the program
'
' Defines to choose only one ADC channel in the OpenADC macro
'
$define ADC_0ANA_ONLY  %0000000000000000         ' All digital
$define ADC_1ANA_ONLY  %0000000000000001         ' analogue: AN0 only
$define ADC_2ANA_ONLY  %0000000000000010         ' analogue: AN1 only
$define ADC_3ANA_ONLY  %0000000000000100         ' analogue: AN2 only
$define ADC_4ANA_ONLY  %0000000000001000         ' analogue: AN3 only
$define ADC_5ANA_ONLY  %0000000000010000         ' analogue: AN4 only
$define ADC_6ANA_ONLY  %0000000000100000         ' analogue: AN5 only
$define ADC_7ANA_ONLY  %0000000001000000         ' analogue: AN6 only
$define ADC_8ANA_ONLY  %0000000010000000         ' analogue: AN7 only
$define ADC_9ANA_ONLY  %0000000100000000         ' analogue: AN8 only
$define ADC_10ANA_ONLY %0000001000000000         ' analogue: AN9 only
$define ADC_11ANA_ONLY %0000010000000000         ' analogue: AN10 only
$define ADC_12ANA_ONLY %0000100000000000         ' analogue: AN11 only
$define ADC_13ANA_ONLY %0001000000000000         ' analogue: AN12 only

'
' Choose the ADC channel to use for the random bit generator
'
$ifndef Random_ADC_Channel                      ' Has the Random_ADC_Channel define been issued?
    $define Random_ADC_Channel 0                ' No. So default to using AN0 for the random bit generator routine
$endif

#ifMacro- _Random_CreateRawBit, _Random_GetBit, _Random_GetByte, _Random_GetWord, _Random_Get
'
' Use some of the microcontroller's SFRs as temporary variables
'
    Dim Random_bTemp1 As PRODL
    Dim Random_bTemp2 As PRODH
    Dim Random_wResult As Word
    Dim Random_bResult As Random_wResult.Byte0
    Dim Random_bIndex As FSR0L
#endIfMacro-

$ifndef False
    $define False 0
$endif

$ifndef True
    $define True 1
$endif

'----------------------------------------------------------------------------------------
    GoTo _Random_Main_                       ' Jump over the subroutines
'----------------------------------------------------------------------------------------
' Create a random bit from an open ADC channel
' Input     : None
' Output    : WREG  holds the random bit value. i.e. 0 or 1
' Notes     : Make sure that no connection is made to the ADC channel being used
'
$define Random_CreateRawBit() _Random_CreateRawBit

_Random_CreateRawBit Macro- \Byte
    GoSub __Random_CreateRawBit
Endm

#ifMacro- _Random_CreateRawBit, _Random_GetBit, _Random_GetByte, _Random_GetWord, _Random_Get
__Random_CreateRawBit:
$if Random_ADC_Channel = 0                  ' Use AN0 as the ADC pin?
    Low PORTA.0                             ' Pull the ADC pin low
    Set PORTA.0                             ' Set the ADC pin high
    Input PORTA.0                           ' Make the ADC pin an input
    ReadADC(ADC_CH0)                        ' Read the ADC value into ADRESL\H

$elseif Random_ADC_Channel = 1              ' Use AN1 as the ADC pin?
    Low PORTA.1                             ' Pull the ADC pin low
    Set PORTA.1                             ' Set the ADC pin high
    Input PORTA.1                           ' Make the ADC pin an input
    ReadADC(ADC_CH1)                        ' Read the ADC value into ADRESL\H

$elseif Random_ADC_Channel = 2              ' Use AN2 as the ADC pin?
    Low PORTA.2                             ' Pull the ADC pin low
    Set PORTA.2                             ' Set the ADC pin high
    Input PORTA.2                           ' Make the ADC pin an input
    ReadADC(ADC_CH2)                        ' Read the ADC value into ADRESL\H

$elseif Random_ADC_Channel = 3              ' Use AN3 as the ADC pin?
    Low PORTA.3                             ' Pull the ADC pin low
    Set PORTA.3                             ' Set the ADC pin high
    Input PORTA.3                           ' Make the ADC pin an input
    ReadADC(ADC_CH3)                        ' Read the ADC value into ADRESL\H

$elseif Random_ADC_Channel = 4              ' Use AN4 as the ADC pin?
    Low PORTA.5                             ' Pull the ADC pin low
    Set PORTA.5                             ' Set the ADC pin high
    Input PORTA.5                           ' Make the ADC pin an input
    ReadADC(ADC_CH4)                        ' Read the ADC value into ADRESL\H
    
$else
    $error Only ADC channels 0 To 4 supported with the Really Random routines    
$endif
    WREG = ADRESL & %00000001               ' Extract the low bit only
    Return
#endIfMacro-
'----------------------------------------------------------------------------------------
' Software whiten bits using Von Neumann algorithm
' Input     : None
' Output    : WREG  holds the random bit value. i.e. 0 or 1
' Notes     : von Neumann, John (1951). "Various techniques used in connection with random digits".
'             National Bureau of Standards Applied Math Series 12:36.
'
$define Random_GetBit() _Random_GetBit

_Random_GetBit Macro- \Byte
    GoSub __Random_GetBit
    Return_Bit STATUS,0
Endm

#ifMacro- _Random_GetBit, _Random_GetByte, _Random_GetWord, _Random_Get
__Random_GetBit:
    While 1 = 1
        Random_CreateRawBit()
        Random_bTemp1 = WREG
        Random_CreateRawBit()
        WREG = WREG << 1
        Random_bTemp1 = Random_bTemp1 | WREG
        If Random_bTemp1 = 1 Then STATUS.0 = 0: Return    ' 1 to 0 transition: log a zero bit
        If Random_bTemp1 = 2 Then STATUS.0 = 1 : Return   ' 0 to 1 transition: log a one bit
    Wend
#endIfMacro-
'----------------------------------------------------------------------------------------
' Create a random byte
' Input     : None
' Output    : WREG holds the random byte value
' Notes     :
'
$define Random_GetByte() _Random_GetByte

_Random_GetByte Macro- \Byte
    GoSub __Random_GetByte
    Return_Wreg
Endm

#ifMacro- _Random_GetByte
__Random_GetByte:
    Random_bIndex = 8
Random_ByteLoop_:
        Random_GetBit()                     ' Get a random bit into the Carry
        Rol Random_bResult                  ' Rotate the Carry bit into the result variable
    Djnz Random_bIndex,Random_ByteLoop_
    WREG = Random_bResult                   ' Transfer the result into the WREG ready to be returned
    Return
#endIfMacro-
'----------------------------------------------------------------------------------------
' Create a random word
' Input     : None
' Output    : Random_wResult holds the random word value
' Notes     :
'
$define Random_GetWord() _Random_GetWord

_Random_GetWord Macro- \Word
    GoSub __Random_GetWord
    Return_Word Random_wResult
Endm

#ifMacro- _Random_GetWord
__Random_GetWord:
    Random_bIndex = 16
Random_WordLoop_:
        Random_GetBit()                     ' Get a random bit into the Carry
        Rol Random_wResult                  ' Rotate the Carry bit into the result variable
    Djnz Random_bIndex,Random_WordLoop_
    Return
#endIfMacro-
'----------------------------------------------------------------------------------------
' Return a random number between 0 and 32767
' Input     : Random_wHowBig holds the (maximum value - 1) to return
' Output    : Random_wResult holds the random value
' Notes     : None
'
$define Random_Get(pHowBig, pResult) '
    Random_wHowBig = pHowBig         '
    _Random_Get                      '
    pResult = Random_wResult

_Random_Get Macro- pHowBig
    GoSub __Random_Get
Endm

#ifMacro _Random_Get
'
' Create some variables only when the Random_Get macro is used
'
    Dim Random_wHowBig As Word
    Dim Random_wTopBit As Word
    Dim Random_wBitPosition As Word

__Random_Get:
    If Random_wHowBig.SWord <= 0 Then
        Random_wResult = 0
        Return
    EndIf

    Random_wTopBit = Random_wHowBig - 1
    Random_wTopBit = Random_wTopBit & Random_wHowBig
    If Random_wTopBit > 0 Then                              ' Range is not a power of 2 so use slow method
        Random_wTopBit = Random_wHowBig - 1
        Random_wBitPosition = Random_wTopBit >> 2
        Random_wTopBit = Random_wTopBit | Random_wBitPosition
        Random_wTopBit = Random_wTopBit + 1
        Random_wTopBit = Random_wTopBit >> 1
Random_TryAgain_:
        Random_wResult = 0
        Random_wBitPosition = Random_wTopBit
        Repeat
            Random_GetBit()                                 ' Generate a bit in the carry flag
            Rol Random_wResult                              ' Rotate it into the resulting variable
            Random_wBitPosition = Random_wBitPosition >> 1
            If Random_wResult >= Random_wHowBig Then        ' Number is over the top limit - start again.
                GoTo Random_TryAgain_
            EndIf
        Until Random_wBitPosition = 0
    Else                                                    ' Special case if pHowBig is a power of 2
        Random_wBitPosition = Random_wHowBig >> 1
        While Random_wBitPosition > 0
            Random_GetBit()                                 ' Generate a bit in the carry flag
            Rol Random_wResult                              ' Rotate it into the resulting variable
            Random_wBitPosition = Random_wBitPosition >> 1
        Wend
    EndIf
    Return
#endIfMacro
'-------------------------------------------------------------------------------------------
_Random_Main_:
'
' Open the ADC  :
'                   Fosc Set For internal RC Oscillator
'                   Right justified for 10-bit operation
'                   Tad value of 2
'                   Vref+ at Vcc : Vref- at Gnd
'                   Make ANx an analogue input
'
$if Random_ADC_Channel = 0                  ' Use AN0 as the ADC pin?
    OpenADC(ADC_FOSC_RC & ADC_RIGHT_JUST & ADC_2_TAD, ADC_REF_VDD_VSS, ADC_0ANA_ONLY)
$elseif Random_ADC_Channel = 1              ' Use AN1 as the ADC pin?
    OpenADC(ADC_FOSC_RC & ADC_RIGHT_JUST & ADC_2_TAD, ADC_REF_VDD_VSS, ADC_1ANA_ONLY)
$elseif Random_ADC_Channel = 2              ' Use AN2 as the ADC pin?
    OpenADC(ADC_FOSC_RC & ADC_RIGHT_JUST & ADC_2_TAD, ADC_REF_VDD_VSS, ADC_2ANA_ONLY)
$elseif Random_ADC_Channel = 3              ' Use AN3 as the ADC pin?
    OpenADC(ADC_FOSC_RC & ADC_RIGHT_JUST & ADC_2_TAD, ADC_REF_VDD_VSS, ADC_3ANA_ONLY)
$elseif Random_ADC_Channel = 4              ' Use AN4 as the ADC pin?
    OpenADC(ADC_FOSC_RC & ADC_RIGHT_JUST & ADC_2_TAD, ADC_REF_VDD_VSS, ADC_4ANA_ONLY)
$endif
    
$endif

